﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Data;
using Arction.Wpf.SemibindableCharting;
using Arction.Wpf.SemibindableCharting.SeriesXY;
using Arction.Wpf.SemibindableCharting.Views.ViewXY;

namespace HeatmapTest
{
    public class MySeriesToIntensityGridSeriesCollectionConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {

            if (!(value is IList<MySeries>))
                return Binding.DoNothing;

            var mySes = (IList<MySeries>)value;
            var iSes = new IntensityGridSeriesCollection();

            foreach (var mSe in mySes)
            {
                var iSe = new IntensityGridSeries();
                iSe.PixelRendering = true;
                iSe.RangeMaxX = 100;
                iSe.RangeMaxY = 100;
                iSe.RangeMinX = 0;
                iSe.RangeMinX = 0;
                iSe.TraceMouseCell = true;
                // The problem is that we're binding series, and the series data is set only once (right here) and the serie is not updated after the original data source is modified on timer. 
                // There are three different solutions at least. 
                // a) Store the reference for the "MySeries" to the real series, and update the series data when when the MySeries data changes
                // b) Same as a but do the reference storing / update using binding => Create binding from MySeries.data -> IntensityGridSeries.data
                // c) Don't handle a list series, bind the data of the predefined series to the datatype of your own choise and make converter for that. 

                var binding = new Binding(nameof(MySeries.Data))
                {
                    Source = mSe,
                    Mode = BindingMode.OneWay,
                    Converter = new DoubleToIntensityPointConverter(),
                };
                BindingOperations.SetBinding(iSe, IntensityGridSeries.DataProperty, binding);

                iSes.Add(iSe);
            };

            return iSes;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }

        //private IntensityPoint[,] ConvertIntesityPoint(double[][] m_data)
        //{
        //    int DataColumnCount = m_data[0].Length;
        //    int DataRowCount = m_data.Length;
        //    var result = new IntensityPoint[DataColumnCount, DataRowCount];
        //    for (int iCol = 0; iCol < DataColumnCount; iCol++)
        //    {
        //        for (int iRow = 0; iRow < DataRowCount; iRow++)
        //        {
        //            result[iCol, iRow] = new IntensityPoint { Value = m_data[iCol][iRow] };
        //        }
        //    }
        //    return result;
        //}
    }


    public class DoubleToIntensityPointConverter : IValueConverter
    {
        // this is called when the source property changes. Note that it's not the same thing as changeing contents of the source property. 
        // Matches older ConvertIntesityPoint method.
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            if (!(value is double[][]))
                return Binding.DoNothing;

            double[][] srcData = (double[][])value;

            // second dimension lenght. Lets assume we always have one item, and all of the second dimensions are of same lenght.
            int jlen = srcData[0].Length;
            // Note that this is not a good practice. 

            // Make new array of IntensityPoints
            IntensityPoint[,] points = new IntensityPoint[srcData.Length, jlen];

            // Populate the list from data. 
            for (int i = 0; i < srcData.Length; i++)
            {
                for (int j = 0; j < jlen; j++)
                {
                    points[i, j] = new IntensityPoint() { Value = srcData[i][j] };
                }
            }

            // return converted array of points
            return points;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

}
